# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - PostMessage Action

    @copyright: 2012 by Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from MoinMoin.PageEditor import PageEditor
from MoinSupport import *
from email.parser import Parser
import os

try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO

Dependencies = ['pages']

class PostMessage:

    "A posted message handler."

    def __init__(self, pagename, request):

        """
        On the page with the given 'pagename', use the given 'request' when
        reading posted messages, modifying the Wiki.
        """

        self.pagename = pagename
        self.request = request
        self.page = Page(request, pagename)

    def do_action(self):
        request = self.request
        content_length = getHeader(request, "Content-Length", "HTTP")
        if content_length:
            content_length = int(content_length)

        # Get the message.

        message_text = StringIO(request.read(content_length))
        self.handle_message(message_text)

    def handle_message(self, message_text):

        "Handle the given 'message_text'."

        request = self.request
        message = Parser().parse(message_text)
        mimetype = message.get_content_type()
        encoding = message.get_content_charset()

        # Detect PGP/GPG-encoded payloads.
        # See: http://tools.ietf.org/html/rfc3156

        if mimetype == "multipart/encrypted" and \
            message.get_param("protocol") == "application/pgp-encrypted":

            try:
                declaration, part = message.get_payload()
            except ValueError:
                writeHeaders(request, "text/plain", getMetadata(self.page), "415 Unsupported Media Type")
                request.write("There must be a declaration and a content part for signed uploads.")
                return

            # Verify the message format.

            if part.get_content_type() != "application/octet-stream":
                writeHeaders(request, "text/plain", getMetadata(self.page), "415 Unsupported Media Type")
                request.write("Encrypted data must be provided as application/octet-stream.")
                return

            # Locate the keyring.

            homedir = getattr(request.cfg, "postmessage_gpg_homedir")
            if not homedir:
                writeHeaders(request, "text/plain", getMetadata(self.page), "415 Unsupported Media Type")
                request.write("Encrypted data cannot currently be understood. Please notify the site administrator.")
                return

            # Decrypt the message text.

            to_child, from_child, error_child = os.popen3(["gpg", "--no-default-keyring", "--homedir", homedir, "--decrypt"])
            to_child.write(part.get_payload())
            to_child.close()
            #print >>open("/tmp/log.txt", "a"), error_child.read()

            # Handle the embedded message.

            try:
                self.handle_plaintext_message(from_child)
            finally:
                from_child.close()
                error_child.close()

        # Reject unsigned payloads.

        else:
            writeHeaders(request, "text/plain", getMetadata(self.page), "415 Unsupported Media Type")
            request.write("Only PGP/GPG-signed payloads are supported.")

    def handle_plaintext_message(self, message_text):

        "Handle the given 'message_text'."

        request = self.request
        message = Parser().parse(message_text)

        # Handle a single part.

        if not message.is_multipart():
            self.handle_message_parts([message], to_replace(message))

        # Handle multiple parts.

        # This can be a collection of updates, with each update potentially being a
        # collection of alternative representations.

        elif is_collection(message):
            for part in message.get_payload():
                if part.is_multipart():
                    self.handle_message_parts(part.get_payload(), to_replace(part))
                else:
                    self.handle_message_parts([part], to_replace(part))

        # Or it can be a collection of alternative representations for a single
        # update.

        else:
            self.handle_message_parts(message.get_payload(), to_replace(message))

        # Default output.

        writeHeaders(request, "text/plain", getMetadata(self.page), "204 No Content")

    def handle_message_parts(self, parts, replace):

        """
        Handle the given message 'parts', replacing the page content if
        'replace' is set to a true value.
        """

        # NOTE: Should either choose preferred content types or somehow retain them
        # NOTE: all but present one at a time.

        body = []

        for part in parts:
            mimetype = part.get_content_type()
            encoding = part.get_content_charset()
            if mimetype == "text/moin":
                body.append(part.get_payload())
                if replace:
                    break

        if not replace:
            body.append(self.page.get_raw_body())

        page_editor = PageEditor(self.request, self.pagename)
        page_editor.saveText("\n\n".join(body), 0)

def is_collection(message):
    return message.get("Update-Type") == "Collection"

def to_replace(message):
    return message.get("Update-Action") == "replace"

# Action function.

def execute(pagename, request):
    PostMessage(pagename, request).do_action()

# vim: tabstop=4 expandtab shiftwidth=4
